@@ -40,6 +40,11 @@ class opensshStatus(StatusMenuItem):
         configReverseOrder = False
         self.configShowWifiHint = True
         
+        try:
+            self.wlanIP = getoutput("/sbin/ifconfig wlan0 | grep 'inet addr:'").strip().split()[1].replace('addr:','')
+        except:
+            self.wlanIP = "None"
+        
         self.parser = SafeConfigParser()
         self.parser.optionxform = str
         if path.isfile("/home/user/.openssh-status-widget"):
@@ -190,6 +195,7 @@ class opensshStatus(StatusMenuItem):
 
         DBusGMainLoop(set_as_default=True)
         bus = dbus.SystemBus()
+        bus.add_signal_receiver(self.handleWLAN, path='/com/nokia/icd', dbus_interface='com.nokia.icd', signal_name='status_changed')
         bus.add_signal_receiver(self.handleDisplay, path='/com/nokia/mce/signal', dbus_interface='com.nokia.mce.signal', signal_name='display_status_ind')
         mceObj = bus.get_object("com.nokia.mce", "/com/nokia/mce/request")
         mceIF = dbus.Interface(mceObj, "com.nokia.mce.request")
@@ -328,6 +334,21 @@ class opensshStatus(StatusMenuItem):
             self.wifiPMOnSec = val
             self.parser.set('wifipm', 'wifiPMOnSec', val)
     
+    def handleWLAN(self, var1, network, status, var4):
+        # network: WLAN_INFRA
+        # status: IDLE / CONNECTING / CONNECTED / DISCONNECTING
+        if network == "WLAN_INFRA":
+            if status == "CONNECTED":
+                try:
+                    self.wlanIP = getoutput("/sbin/ifconfig wlan0 | grep 'inet addr:'").strip().split()[1].replace('addr:','')
+                except:
+                    self.wlanIP = "None"
+                if self.wlanIP != "None" and self.keepWifiPMOff:
+                    system("sudo openssh-status.sh wlan_started")
+            if status == "IDLE":
+                self.wlanIP = "None"
+                system("sudo openssh-status.sh wlan_stopped")
+    
     def handleDisplay(self, displayStatus):
         if displayStatus == "on" or displayStatus == "dimmed":
             self.loops = 0
@@ -463,48 +484,72 @@ class opensshStatus(StatusMenuItem):
         self.sshConnected = int(getoutput("netstat -t | grep -c :ssh[[:space:]]*[0-9].*ESTABLISHED"))
         
     def sshAllUsers(self):
+        self.sshConnected = int(getoutput("netstat -t | grep -c :ssh[[:space:]]*[0-9].*ESTABLISHED"))
         sshNetstat = getoutput("netstat -t | grep :ssh.*ESTABLISHED").split("\n")
         
-        countAllWLAN = 0
-        for line in sshNetstat:
-            if self.wlanIP in line:
-                countAllWLAN += 1
         countInWLAN = 0
-        if countAllWLAN:
+        countOutWLAN = 0
+        if self.wlanIP != "None":
+            countAllWLAN = 0
             for line in sshNetstat:
-                element = " ".join(line.split()).split()[3]
-                if self.wlanIP + ":ssh" in element:
-                    countInWLAN += 1
-        countOutWLAN = countAllWLAN - countInWLAN
+                if self.wlanIP in line:
+                    countAllWLAN += 1
+            if countAllWLAN:
+                for line in sshNetstat:
+                    element = " ".join(line.split()).split()[3]
+                    if self.wlanIP + ":ssh" in element:
+                        countInWLAN += 1
+            countOutWLAN = countAllWLAN - countInWLAN
+            if countInWLAN != self.labelCountInWLAN:
+                self.labelCountInWLAN = countInWLAN
+                self.labelInWLAN.set_label(str(countInWLAN) + "  WLAN")
+            if countOutWLAN != self.labelCountOutWLAN:
+                self.labelCountOutWLAN = countOutWLAN
+                self.labelOutWLAN.set_label(str(countOutWLAN) + "  WLAN")
+        else:
+            self.labelInWLAN.set_label("?  WLAN")
+            self.labelOutWLAN.set_label("?  WLAN")
         
-        countAllUSB = 0
-        for line in sshNetstat:
-            if self.usbIP in line:
-                countAllUSB += 1
         countInUSB = 0
-        if countAllUSB:
+        countOutUSB = 0
+        if self.usbIP != "None":
+            countAllUSB = 0
             for line in sshNetstat:
-                element = " ".join(line.split()).split()[3]
-                if self.usbIP + ":ssh" in element:
-                    countInUSB += 1
-        countOutUSB = countAllUSB - countInUSB
+                if self.usbIP in line:
+                    countAllUSB += 1
+            if countAllUSB:
+                for line in sshNetstat:
+                    element = " ".join(line.split()).split()[3]
+                    if self.usbIP + ":ssh" in element:
+                        countInUSB += 1
+            countOutUSB = countAllUSB - countInUSB
+            if countInUSB != self.labelCountInUSB:
+                self.labelCountInUSB = countInUSB
+                self.labelInUSB.set_label(str(countInUSB) + "  USB")
+            if countOutUSB != self.labelCountOutUSB:
+                self.labelCountOutUSB = countOutUSB
+                self.labelOutUSB.set_label(str(countOutUSB) + "  USB")
+        else:
+            self.labelInUSB.set_label("?  USB")
+            self.labelOutUSB.set_label("?  USB")
         
-        self.sshConnected = countInWLAN + countInUSB
-        
-        if countInWLAN != self.labelCountInWLAN:
-            self.labelCountInWLAN = countInWLAN
-            self.labelInWLAN.set_label(str(countInWLAN) + "  WLAN")
-        if countInUSB != self.labelCountInUSB:
-            self.labelCountInUSB = countInUSB
-            self.labelInUSB.set_label(str(countInUSB) + "  USB")
-        if countOutWLAN != self.labelCountOutWLAN:
-            self.labelCountOutWLAN = countOutWLAN
-            self.labelOutWLAN.set_label(str(countOutWLAN) + "  WLAN")
-        if countOutUSB != self.labelCountOutUSB:
-            self.labelCountOutUSB = countOutUSB
-            self.labelOutUSB.set_label(str(countOutUSB) + "  USB")
-    
+        countInOther = self.sshConnected - countInWLAN - countInUSB
+        countOutOther = len(sshNetstat) - self.sshConnected - countOutWLAN - countOutUSB
         
+        if countInOther:
+            if countInOther != self.labelCountInOther:
+                self.labelCountInOther = countInOther
+                self.labelInOther.set_label(str(countInOther) + "  Other")
+        else:
+            self.labelInOther.set_label("")
+            
+        if countOutOther:
+            if countOutOther != self.labelCountOutOther:
+                self.labelCountOutOther = countOutOther
+                self.labelOutOther.set_label(str(countOutOther) + "  Other")
+        else:
+            self.labelOutOther.set_label("")
+            
     def sshTracker(self):
         if self.trackingEnabled:
             # Smart frequency
@@ -615,19 +660,25 @@ class opensshStatus(StatusMenuItem):
     def button2Clicked(self, dummy):
         self.get_toplevel().hide()
         
-        wlanIP = getoutput("/sbin/ifconfig wlan0 | grep 'inet addr:'").strip().split(' ')[1].replace('addr:','')
-        if  not int(getoutput("netstat -t | grep :ssh.*ESTABLISHED | grep -c " + wlanIP)):
-            self.printInfo("Wifi PM is kept off for 30 sec.\nGood time to connect...", 7)
-            system("sudo openssh-status.sh wifipmoff_once")
-        elif int(getoutput("ps aux | grep -v grep  | grep -c /usr/sbin/osw-power-management")):
-            if self.keepWifiPMOff == True and self.toggleWifiPM == False:
-                self.printInfo("Wifi PM is off anyway.\nNo need to keep it off for 30 sec.", 7)
+        try:
+            wlanIP = getoutput("/sbin/ifconfig wlan0 | grep 'inet addr:'").strip().split()[1].replace('addr:','')
+        except:
+            wlanIP = "None"
+        if wlanIP == "None":
+            self.printInfo("No WLAN was detected.\nWifi PM is <b>not</b> kept off for 30 sec.", 7)
+        else:
+            if  not int(getoutput("netstat -t | grep :ssh.*ESTABLISHED | grep -c " + wlanIP)):
+                self.printInfo("Wifi PM is kept off for 30 sec.\nGood time to connect...", 7)
+                system("sudo openssh-status.sh wifipmoff_once")
+            elif int(getoutput("ps aux | grep -v grep  | grep -c /usr/sbin/osw-power-management")):
+                if self.keepWifiPMOff == True and self.toggleWifiPM == False:
+                    self.printInfo("Wifi PM is off anyway.\nNo need to keep it off for 30 sec.", 7)
+                else:
+                    self.printInfo("Wifi PM is kept off for 30 sec.\nGood time to connect...", 7)
+                    system("sudo openssh-status.sh wifipmoff_once")
             else:
                 self.printInfo("Wifi PM is kept off for 30 sec.\nGood time to connect...", 7)
                 system("sudo openssh-status.sh wifipmoff_once")
-        else:
-            self.printInfo("Wifi PM is kept off for 30 sec.\nGood time to connect...", 7)
-            system("sudo openssh-status.sh wifipmoff_once")
 
     def button2Pressed(self, dummy):
         self.get_toplevel().hide()        
@@ -874,10 +925,6 @@ class opensshStatus(StatusMenuItem):
             vbox2.set_border_width(8)
             
             if self.showAllConnections:
-                try:
-                    self.wlanIP = getoutput("/sbin/ifconfig wlan0 | grep 'inet addr:'").strip().split()[1].replace('addr:','')
-                except:
-                    self.wlanIP = "None"
                 try:
                     self.usbIP = getoutput("/sbin/ifconfig usb0 | grep 'inet addr:'").strip().split()[1].replace('addr:','')
                 except:
@@ -890,10 +937,14 @@ class opensshStatus(StatusMenuItem):
                 self.labelInWLAN.set_alignment(1, 0)
                 self.labelInUSB = gtk.Label("0  USB")
                 self.labelInUSB.set_alignment(1, 0)
+                self.labelInOther = gtk.Label("")
+                self.labelInOther.set_alignment(1, 0)
                 self.labelOutWLAN = gtk.Label("0  WLAN")
                 self.labelOutWLAN.set_alignment(1, 0)
                 self.labelOutUSB = gtk.Label("0  USB")
                 self.labelOutUSB.set_alignment(1, 0)
+                self.labelOutOther = gtk.Label("")
+                self.labelOutOther.set_alignment(1, 0)
                 
                 if self.trackingEnabled:
                     self.showAllDialog = True
@@ -905,19 +956,23 @@ class opensshStatus(StatusMenuItem):
                     vboxLabel1 = gtk.VBox(False, 0)
                     vboxLabel2 = gtk.VBox(False, 0)
                     vboxLabel3 = gtk.VBox(False, 0)
+                    vboxLabel4 = gtk.VBox(False, 0)
                     vboxLabel1.pack_start(labelIn, False, False, 0)
                     vboxLabel1.pack_start(labelOut, False, False, 0)
                     vboxLabel2.pack_start(self.labelInWLAN, False, False, 0)
                     vboxLabel2.pack_start(self.labelOutWLAN, False, False, 0)
                     vboxLabel3.pack_start(self.labelInUSB, False, False, 0)
                     vboxLabel3.pack_start(self.labelOutUSB, False, False, 0)
+                    vboxLabel4.pack_start(self.labelInOther, False, False, 0)
+                    vboxLabel4.pack_start(self.labelOutOther, False, False, 0)
                     spacerLabel = gtk.Label()
-                    hboxLabel23 = gtk.HBox(False, 0)
-                    hboxLabel23.pack_start(vboxLabel2, False, False, 0)
-                    hboxLabel23.pack_start(vboxLabel3, False, False, 32)
-                    hboxLabel23.pack_start(spacerLabel, True, True, 0)
+                    hboxLabel234 = gtk.HBox(False, 0)
+                    hboxLabel234.pack_start(vboxLabel2, False, False, 0)
+                    hboxLabel234.pack_start(vboxLabel3, False, False, 32)
+                    hboxLabel234.pack_start(vboxLabel4, False, False, 0)
+                    hboxLabel234.pack_start(spacerLabel, True, True, 0)
                     hboxConnections.pack_start(vboxLabel1, True, True, 0)
-                    hboxConnections.pack_start(hboxLabel23, True, True, 0)
+                    hboxConnections.pack_start(hboxLabel234, True, True, 0)
                     spaceLeftLabel = gtk.Label()
                     spaceRightLabel = gtk.Label()
                     hbox123 = gtk.HBox(False, 0)
@@ -932,18 +987,22 @@ class opensshStatus(StatusMenuItem):
                     vboxLabel1 = gtk.VBox(False, 0)
                     vboxLabel2 = gtk.VBox(False, 0)
                     vboxLabel3 = gtk.VBox(False, 0)
+                    vboxLabel4 = gtk.VBox(False, 0)
                     vboxLabel1.pack_start(labelIn, False, False, 0)
                     vboxLabel1.pack_start(labelOut, False, False, 0)
                     vboxLabel2.pack_start(self.labelInWLAN, False, False, 0)
                     vboxLabel2.pack_start(self.labelOutWLAN, False, False, 0)
                     vboxLabel3.pack_start(self.labelInUSB, False, False, 0)
                     vboxLabel3.pack_start(self.labelOutUSB, False, False, 0)
+                    vboxLabel4.pack_start(self.labelInOther, False, False, 0)
+                    vboxLabel4.pack_start(self.labelOutOther, False, False, 0)
                     refreshButton = hildon.GtkButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
                     refreshButton.set_label("Refresh")
-                    hboxConnections = gtk.HBox(False, 60)
-                    hboxConnections.pack_start(vboxLabel1, False, False, 0)
-                    hboxConnections.pack_start(vboxLabel2, False, False, 0)
-                    hboxConnections.pack_start(vboxLabel3, False, False, 0)
+                    hboxConnections = gtk.HBox(False, 0)
+                    hboxConnections.pack_start(vboxLabel1, True, False, 0)
+                    hboxConnections.pack_start(vboxLabel2, True, False, 0)
+                    hboxConnections.pack_start(vboxLabel3, True, False, 0)
+                    hboxConnections.pack_start(vboxLabel4, True, False, 0)
                     hboxConnections.pack_start(refreshButton, True, True, 0)
                     spaceLeftLabel = gtk.Label()
                     spaceRightLabel = gtk.Label()
@@ -1008,40 +1067,69 @@ class opensshStatus(StatusMenuItem):
         self.showAllDialog = False
             
     def sshCountAll(self, dummy):
+        self.sshUsers()
         sshNetstat = getoutput("netstat -t | grep :ssh.*ESTABLISHED").split("\n")
         
-        countAllWLAN = 0
-        for line in sshNetstat:
-            if self.wlanIP in line:
-                countAllWLAN += 1
         countInWLAN = 0
-        if countAllWLAN:
+        countOutWLAN = 0
+        if self.wlanIP != "None":
+            countAllWLAN = 0
             for line in sshNetstat:
-                element = " ".join(line.split()).split()[3]
-                if self.wlanIP + ":ssh" in element:
-                    countInWLAN += 1
-        countOutWLAN = countAllWLAN - countInWLAN
-        self.labelCountInWLAN = countInWLAN
-        self.labelCountOutWLAN = countOutWLAN
+                if self.wlanIP in line:
+                    countAllWLAN += 1
+            if countAllWLAN:
+                for line in sshNetstat:
+                    element = " ".join(line.split()).split()[3]
+                    if self.wlanIP + ":ssh" in element:
+                        countInWLAN += 1
+            countOutWLAN = countAllWLAN - countInWLAN
+            self.labelCountInWLAN = countInWLAN
+            self.labelCountOutWLAN = countOutWLAN
+            self.labelInWLAN.set_label(str(countInWLAN) + "  WLAN")
+            self.labelOutWLAN.set_label(str(countOutWLAN) + "  WLAN")
+        else:
+            self.labelCountInWLAN = -1
+            self.labelCountOutWLAN = -1
+            self.labelInWLAN.set_label("?  WLAN")
+            self.labelOutWLAN.set_label("?  WLAN")
         
-        countAllUSB = 0
-        for line in sshNetstat:
-            if self.usbIP in line:
-                countAllUSB += 1
         countInUSB = 0
-        if countAllUSB:
+        countOutUSB = 0
+        if self.usbIP != "None":
+            countAllUSB = 0
             for line in sshNetstat:
-                element = " ".join(line.split()).split()[3]
-                if self.usbIP + ":ssh" in element:
-                    countInUSB += 1
-        countOutUSB = countAllUSB - countInUSB
-        self.labelCountInUSB = countInUSB
-        self.labelCountOutUSB = countOutUSB
+                if self.usbIP in line:
+                    countAllUSB += 1
+            if countAllUSB:
+                for line in sshNetstat:
+                    element = " ".join(line.split()).split()[3]
+                    if self.usbIP + ":ssh" in element:
+                        countInUSB += 1
+            countOutUSB = countAllUSB - countInUSB
+            self.labelCountInUSB = countInUSB
+            self.labelCountOutUSB = countOutUSB
+            self.labelInUSB.set_label(str(countInUSB) + "  USB")
+            self.labelOutUSB.set_label(str(countOutUSB) + "  USB")
+        else:
+            self.labelCountInUSB = -1
+            self.labelCountOutUSB = -1
+            self.labelInUSB.set_label("?  USB")
+            self.labelOutUSB.set_label("?  USB")
+            
+        countInOther = self.sshConnected - countInWLAN - countInUSB
+        countOutOther = len(sshNetstat) - self.sshConnected - countOutWLAN - countOutUSB
+        self.labelCountInOther = countInOther
+        self.labelCountOutOther = countOutOther
         
-        self.labelInWLAN.set_label(str(countInWLAN) + "  WLAN")
-        self.labelInUSB.set_label(str(countInUSB) + "  USB")
-        self.labelOutWLAN.set_label(str(countOutWLAN) + "  WLAN")
-        self.labelOutUSB.set_label(str(countOutUSB) + "  USB")
+        if countInOther:
+            self.labelInOther.set_label(str(countInOther) + "  Other")
+        else:
+            self.labelInOther.set_label("")
+            
+        if countOutOther:
+            self.labelOutOther.set_label(str(countOutOther) + "  Other")
+        else:
+            self.labelOutOther.set_label("")
     
     def buttonSettings_clicked(self, dummy):
         self.dialog.destroy()
